//
//  SCINetworkAgent.h
//  ComNetKit
//
//  Created by Filipi Nascimento Silva on 06/17/2008.
//  Copyright 2008 Mega X Software. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <SciObjects/SciObjects.h>
#import <ComNetKit/ComNetKit.h>

typedef struct _SCIAgentInformation {
	BOOL* visitedVertices;
	BOOL finished;
	NSUInteger walkCount;
} SCIAgentInformation;


SCIAgentInformation newSCIAgentInformation();

void deleteSCIAgentInformation(SCIAgentInformation information);

	
	
@interface SCINetworkAgentsGroup : NSObject {
@public
	SCINetwork* __relatedNetwork;
	NSUInteger __countAgents;
	NSInteger __currentOrderPointer;
	SCINumberArray* __visitFrequencyOfVertices;
	SCINumberArray* __orderOfVisitedVertices;
	SCINumberArray* __agentsCurrentVertices;
	SCINumberArray* __agentsOrderCounter;
	NSMutableArray* __agentVisitFrequencyOfVerticesList;
	NSMutableArray* __agentOrderOfVisitedVerticesList;
	NSInteger _currentProgress;
	NSInteger _totalWork;
	BOOL __shouldWait;
	BOOL __shouldResume;
	BOOL __isWaiting;
	BOOL __isWorking;
	NSUInteger __synchronizationSteps;
	NSMutableDictionary* agentsProperties;
	SCIAgentInformation __walkInformation;
}


-initAtNetwork:(SCINetwork*)relatedNetwork initialVertices:(NSArray*)initialVertices;
-initAtNetwork:(SCINetwork*)relatedNetwork initialVerticesIndices:(SCINumberArray*)initialVerticesIndices;


/*
-initAtNetwork:(SCINetwork*)relatedNetwork initialVertices:(NSArray*)initialVertices 
	selectNextVertexFunction:(NSUInteger (*callback)(NSUInteger currentVertex,
													 float* adjacencyMatrix, 
													 NSUInteger* visitFrequencyOfVertices,
													 NSUInteger* orderOfVisitedVertices,
													 NSUInteger numberOfNodes))callbackFunction;
*/


-(NSArray*) agentsCurrentVerticesList;
-(NSArray*) tellAgentsToWalkToNextVertices;

-(SCIVertex*) currentVertexOfAgent:(NSUInteger)agentIndex;
-(SCIVertex*) walkNextVertexForAgent:(NSUInteger)agentIndex;

-(NSArray*) walkForSteps:(NSUInteger)steps;
-(NSArray*) walkForSteps:(NSUInteger)steps returnPath:(BOOL)returnPath;
-(NSArray*) walkForSteps:(NSUInteger)steps returnPath:(BOOL)returnPath untilReach:(float)percentage;

-(void) setAgentsCurrentVertices:(NSArray*)newCurrentVertices;
-(void) setAgentsCurrentVerticesIndices:(SCINumberArray*)newCurrentVerticesIndices;
-(void) setCurrentVertex:(SCIVertex*)aVertex forAgent:(NSUInteger)agentIndex;

-(NSMutableDictionary*) properties;
-(NSMutableDictionary*) propertiesForAgent:(NSUInteger)agentIndex;


-(void) setProperties:(NSMutableDictionary*) properties;
-(void) setProperties:(NSDictionary*) properties forAgent:(NSUInteger)agentIndex;

-(NSArray*) propertyValuesForKey:(NSString*)aKey;
-(void) setPropertyValues:(NSArray*)value forKey:(NSString*)aKey;

-(id) propertyValueForAgent:(NSUInteger)agentIndex forKey:(NSString*)aKey ;
-(void) setPropertyValue:(id)value forKey:(NSString*)aKey forAgent:(NSUInteger)agentIndex;

-(SCINumberArray*) globalVisitFrequencyOfVertices;
-(SCINumberArray*) globalOrderOfVisitedVertices;

-(SCINumberArray*) visitFrequencyOfVerticesForAgent:(NSInteger)agentIndex;
-(SCINumberArray*) orderOfVisitedVerticesForAgent:(NSInteger)agentIndex;

-(void) setSynchronizationSteps:(NSUInteger)steps;
-(NSUInteger) synchronizationSteps;

-(NSInteger) countAgents;
-(BOOL) reachedWalkPercentage;
-(NSUInteger) walkCount;
-(BOOL) flushWalkHistory;

-(NSInteger) currentProgress;
-(NSInteger) currentWork;

-(void) pauseWork;
-(void) resumeWork;
-(BOOL) isWaiting;
-(BOOL) isWorking;
-(void) cancelWork;


@end




#pragma mark "Dynamic Network"


@interface SCINetworkDynamicAgentsGroup : SCINetworkAgentsGroup {
@public
	SCINetwork* __agentsNetwork;
	float* __agentsErrorsPointer;
	NSUInteger* __agentsMemorySizesPointer;
	NSInteger* __agentsMemoryLastPosition;
	NSInteger* __agentsMemoryTransferPosition;
	NSUInteger* __agentsMemoryCurrentPosition;
	NSUInteger** __agentMemoryPointer;
	
	NSUInteger** __agentsVertexVisitedPointer;
	BOOL firstWalk;
}


-initAtNetwork:(SCINetwork*)relatedNetwork initialVertices:(NSArray*)initialVertices agentsNetwork:(SCINetwork*)agentsNetwork agentsErrors:(SCINumberArray*)agentsErrors agentsMemory:(SCINumberArray*)agentsMemory;
-initAtNetwork:(SCINetwork*)relatedNetwork initialVerticesIndices:(SCINumberArray*)initialVerticesIndices agentsNetwork:(SCINetwork*)agentsNetwork agentsErrors:(SCINumberArray*)agentsErrors agentsMemory:(SCINumberArray*)agentsMemory;


@end
