//
//  SCINumberArray.h
//  ComNetKit
//
//  Created by Filipi Nascimento Silva on 02/17/2008.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SciObjects_consts.h"

typedef enum {
	SCIUndefinedNumberArrayType			=	-1,
	SCIBoolNumberArrayType				=	 1,
	SCICharNumberArrayType				=	 2,
	SCIUnsignedCharNumberArrayType		=	 3,
	SCIShortNumberArrayType				=	 4,
	SCIUnsignedShortNumberArrayType		=	 5,
    SCIIntNumberArrayType				=	 6,
	SCIUnsignedIntNumberArrayType		=	 7,
	SCIIntegerNumberArrayType			=	 8,
	SCIUnsignedIntegerNumberArrayType	=	 9,
	SCILongNumberArrayType				=	10,
	SCIUnsignedLongNumberArrayType		=	11,
	SCILongLongNumberArrayType			=	12,
	SCIUnsignedLongLongNumberArrayType	=	13,
	SCIFloatNumberArrayType				=	14,
	SCIDoubleNumberArrayType			=	15
} SCINumberArrayType;

@class SCINumberArray, SCINumberArray_initializer, SCIExpression;

//additions to NSArray
@interface NSArray (SCISubArrayAdditions)
- (NSArray *)subarrayWithIndices:(SCINumberArray*)indicesArray;
@end

@protocol SCIEditableArray
	-(void)replaceObject:(id)object atIndex:(NSInteger)index;
	-(void) blockEditing;
	-(BOOL) isEditable;
	-editableCopy;
@end

@interface NSMutableArray (SCISetAdditions) <SCIEditableArray>
@end

@interface NSArray (SCISetAdditions) <SCIEditableArray>
@end

@interface SCINumberArray : NSMutableArray <SCIEditableArray>{
}


-initWithBoolArray:(BOOL*)boolValues count:(NSUInteger)count;
-initWithCharArray:(char*)charValues count:(NSUInteger)count;
-initWithShortArray:(short*)shortValues count:(NSUInteger)count;
-initWithIntArray:(int*)intValues count:(NSUInteger)count;
-initWithLongArray:(long*)longValues count:(NSUInteger)count;
-initWithLongLongArray:(long long*)longLongValues count:(NSUInteger)count;
-initWithFloatArray:(float*)floatValues count:(NSUInteger)count;
-initWithDoubleArray:(double*)doubleValues count:(NSUInteger)count;
-initWithIntegerArray:(NSInteger*)integerValues count:(NSUInteger)count;

-initWithUnsignedCharArray:(unsigned char*)uCharValues count:(NSUInteger)count;
-initWithUnsignedShortArray:(unsigned short*)uShortValues count:(NSUInteger)count;
-initWithUnsignedIntArray:(unsigned int*)uIntValues count:(NSUInteger)count;
-initWithUnsignedLongArray:(unsigned long*)uLongValues count:(NSUInteger)count;
-initWithUnsignedLongLongArray:(unsigned long long*)uLongLongValues count:(NSUInteger)count;
-initWithUnsignedIntegerArray:(NSUInteger*)uIntegerValues count:(NSUInteger)count;


//MutableSCINumberArrays:
-initBoolArrayWithCapacity:(NSUInteger)capacity;
-initCharArrayWithCapacity:(NSUInteger)capacity;
-initShortArrayWithCapacity:(NSUInteger)capacity;
-initIntArrayWithCapacity:(NSUInteger)capacity;
-initLongArrayWithCapacity:(NSUInteger)capacity;
-initLongLongArrayWithCapacity:(NSUInteger)capacity;
-initFloatArrayWithCapacity:(NSUInteger)capacity;
-initDoubleArrayWithCapacity:(NSUInteger)capacity;
-initIntegerArrayWithCapacity:(NSUInteger)capacity;

-initUnsignedCharArrayWithCapacity:(NSUInteger)capacity;
-initUnsignedShortArrayWithCapacity:(NSUInteger)capacity;
-initUnsignedIntArrayWithCapacity:(NSUInteger)capacity;
-initUnsignedLongArrayWithCapacity:(NSUInteger)capacity;
-initUnsignedLongLongArrayWithCapacity:(NSUInteger)capacity;
-initUnsignedIntegerArrayWithCapacity:(NSUInteger)capacity;
-initWithCapacity:(NSUInteger)capacity;


//Scalar Methods:
-initWithBoolScalar:(BOOL)boolValue count:(NSUInteger)count;
-initWithCharScalar:(char)charValue count:(NSUInteger)count;
-initWithShortScalar:(short)shortValue count:(NSUInteger)count;
-initWithIntScalar:(int)intValue count:(NSUInteger)count;
-initWithLongScalar:(long)longValue count:(NSUInteger)count;
-initWithLongLongScalar:(long long)longLongValue count:(NSUInteger)count;
-initWithFloatScalar:(float)floatValue count:(NSUInteger)count;
-initWithDoubleScalar:(double)doubleValue count:(NSUInteger)count;
-initWithIntegerScalar:(NSInteger)integerValue count:(NSUInteger)count;

-initWithUnsignedCharScalar:(unsigned char)uCharValue count:(NSUInteger)count;
-initWithUnsignedShortScalar:(unsigned short)uShortValue count:(NSUInteger)count;
-initWithUnsignedIntScalar:(unsigned int)uIntValue count:(NSUInteger)count;
-initWithUnsignedLongScalar:(unsigned long)uLongValue count:(NSUInteger)count;
-initWithUnsignedLongLongScalar:(unsigned long long)uLongLongValue count:(NSUInteger)count;
-initWithUnsignedIntegerScalar:(NSUInteger)uIntegerValue count:(NSUInteger)count;




-initWithArray:(NSArray *) array;
-initWithNumberArray:(SCINumberArray *) numberArray;

-initWithArray:(NSArray *) array count:(NSUInteger)count;
-initWithNumberArray:(SCINumberArray *) numberArray count:(NSUInteger)count;

-initWithNumber:(NSNumber*)number count:(NSUInteger)count;


-initByMergingNumberArrays:(NSArray *) numberArrays;
-initByMergingNumberArrays:(NSArray *) numberArrays type:(SCINumberArrayType)type;

-initWithNumberArray:(SCINumberArray *) numberArray range:(NSRange)range;
-initWithNumberArray:(SCINumberArray *) numberArray range:(NSRange)range type:(SCINumberArrayType)type;

-initWithNumberArray:(SCINumberArray *) numberArray indices:(SCINumberArray*)indicesArray;
-initWithNumberArray:(SCINumberArray *) numberArray indices:(SCINumberArray*)indicesArray type:(SCINumberArrayType)type;








+(SCINumberArray *)numberArrayWithBoolArray:(BOOL*)boolValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithCharArray:(char*)charValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithShortArray:(short*)shortValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithIntArray:(int*)intValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithLongArray:(long*)longValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithLongLongArray:(long long*)longLongValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithFloatArray:(float*)floatValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithDoubleArray:(double*)doubleValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithIntegerArray:(NSInteger*)integerValues count:(NSUInteger)count;

+(SCINumberArray *)numberArrayWithUnsignedCharArray:(unsigned char*)uCharValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithUnsignedShortArray:(unsigned short*)uShortValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithUnsignedIntArray:(unsigned int*)uIntValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithUnsignedLongArray:(unsigned long*)uLongValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithUnsignedLongLongArray:(unsigned long long*)uLongLongValues count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithUnsignedIntegerArray:(NSUInteger*)uIntegerValues count:(NSUInteger)count;


//MutableSCINumberArrays:
+(SCINumberArray *)boolNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)charNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)shortNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)intNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)longNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)longLongNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)floatNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)doubleNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)integerNumberArrayWithCapacity:(NSUInteger)capacity;

+(SCINumberArray *)unsignedCharNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)unsignedShortNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)unsignedIntNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)unsignedLongNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)unsignedLongLongNumberArrayWithCapacity:(NSUInteger)capacity;
+(SCINumberArray *)unsignedIntegerNumberArrayWithCapacity:(NSUInteger)capacity;

//Scalar Methods:
+(SCINumberArray *)boolArrayWithScalar:(BOOL)boolValue count:(NSUInteger)count;
+(SCINumberArray *)charArrayWithScalar:(char)charValue count:(NSUInteger)count;
+(SCINumberArray *)shortArrayWithScalar:(short)shortValue count:(NSUInteger)count;
+(SCINumberArray *)intArrayWithScalar:(int)intValue count:(NSUInteger)count;
+(SCINumberArray *)longArrayWithScalar:(long)longValue count:(NSUInteger)count;
+(SCINumberArray *)longLongArrayWithScalar:(long long)longLongValue count:(NSUInteger)count;
+(SCINumberArray *)floatArrayWithScalar:(float)floatValue count:(NSUInteger)count;
+(SCINumberArray *)doubleArrayWithScalar:(double)doubleValue count:(NSUInteger)count;
+(SCINumberArray *)integerArrayWithScalar:(NSInteger)integerValue count:(NSUInteger)count;

+(SCINumberArray *)unsignedCharArrayWithScalar:(unsigned char)uCharValue count:(NSUInteger)count;
+(SCINumberArray *)unsignedShortArrayWithScalar:(unsigned short)uShortValue count:(NSUInteger)count;
+(SCINumberArray *)unsignedIntArrayWithScalar:(unsigned int)uIntValue count:(NSUInteger)count;
+(SCINumberArray *)unsignedLongArrayWithScalar:(unsigned long)uLongValue count:(NSUInteger)count;
+(SCINumberArray *)unsignedLongLongArrayWithScalar:(unsigned long long)uLongLongValue count:(NSUInteger)count;
+(SCINumberArray *)unsignedIntegerArrayWithScalar:(NSUInteger)uIntegerValue count:(NSUInteger)count;




+(SCINumberArray *)numberArrayWithArray:(NSArray *) array;
+(SCINumberArray *)numberArrayWithNumberArray:(SCINumberArray *) numberArray;
+(SCINumberArray *)numberArrayWithNumber:(NSNumber*)number count:(NSUInteger)count;

+(SCINumberArray *)numberArrayWithArray:(NSArray *) array count:(NSUInteger)count;
+(SCINumberArray *)numberArrayWithNumberArray:(SCINumberArray *) numberArray count:(NSUInteger)count;

+(SCINumberArray *)numberArrayByMergingNumberArrays:(NSArray *) numberArrays;
+(SCINumberArray *)numberArrayByMergingNumberArrays:(NSArray *) numberArrays type:(SCINumberArrayType)type;

+(SCINumberArray *)numberArrayWithNumberArray:(NSArray *) numberArrays range:(NSRange)range;
+(SCINumberArray *)numberArrayWithNumberArray:(NSArray *) numberArrays range:(NSRange)range type:(SCINumberArrayType)type;


+(SCINumberArray *)numberArrayWithNumberArray:(SCINumberArray *) numberArray indices:(SCINumberArray*)indicesArray;
+(SCINumberArray *)numberArrayWithNumberArray:(SCINumberArray *) numberArray indices:(SCINumberArray*)indicesArray type:(SCINumberArrayType)type;


-(void)getComparisonsWithNumberArray:(SCINumberArray *)otherNumberArray inComparasionBuffer:(NSComparisonResult*)comparisonResults;

-(SCINumberArray *)equalitiesWithNumberArray:(SCINumberArray *)otherNumberArray;
-(BOOL)isEqual:otherNumberArray;
-(BOOL)isEqualToNumberArray:(SCINumberArray *)otherNumberArray;

-(void)getCharValues:(char*) charBuffer;
-(void)getShortValues:(short*) shortBuffer;
-(void)getIntValues:(int*) intBuffer;
-(void)getLongValues:(long*) longBuffer;
-(void)getLongLongValues:(long long*) longLongBuffer;
-(void)getFloatValues:(float*) floatBuffer;
-(void)getDoubleValues:(double*) doubleBuffer;
-(void)getBoolValues:(BOOL*) boolBuffer;
-(void)getDecimalValues:(NSDecimal*) decimalBuffer;
-(void)getIntegerValues:(NSInteger*) integerBuffer;

-(void)getUnsignedCharValues:(unsigned char*) ucharBuffer;
-(void)getUnsignedShortValues:(unsigned short*) ushortBuffer;
-(void)getUnsignedIntValues:(unsigned int*) uintBuffer;
-(void)getUnsignedLongValues:(unsigned long*) ulongBuffer;
-(void)getUnsignedLongLongValues:(unsigned long long*) ulonglongBuffer;
-(void)getUnsignedIntegerValues:(NSUInteger*) uintegerBuffer;


-(void)getCharValues:(char*) charBuffer count:(NSUInteger) count;
-(void)getShortValues:(short*) shortBuffer count:(NSUInteger) count;
-(void)getIntValues:(int*) intBuffer count:(NSUInteger) count;
-(void)getLongValues:(long*) longBuffer count:(NSUInteger) count;
-(void)getLongLongValues:(long long*) longLongBuffer count:(NSUInteger) count;
-(void)getFloatValues:(float*) floatBuffer count:(NSUInteger) count;
-(void)getDoubleValues:(double*) doubleBuffer count:(NSUInteger) count;
-(void)getBoolValues:(BOOL*) boolBuffer count:(NSUInteger) count;
-(void)getDecimalValues:(NSDecimal*) decimalBuffer count:(NSUInteger) count;
-(void)getIntegerValues:(NSInteger*) integerBuffer count:(NSUInteger) count;

-(void)getUnsignedCharValues:(unsigned char*) ucharBuffer count:(NSUInteger) count;
-(void)getUnsignedShortValues:(unsigned short*) ushortBuffer count:(NSUInteger) count;
-(void)getUnsignedIntValues:(unsigned int*) uintBuffer count:(NSUInteger) count;
-(void)getUnsignedLongValues:(unsigned long*) ulongBuffer count:(NSUInteger) count;
-(void)getUnsignedLongLongValues:(unsigned long long*) ulonglongBuffer count:(NSUInteger) count;
-(void)getUnsignedIntegerValues:(NSUInteger*) uintegerBuffer count:(NSUInteger) count;

//getters:
-(char)charValueAtIndex:(NSUInteger) index;
-(short)shortValueAtIndex:(NSUInteger) index;
-(int)intValueAtIndex:(NSUInteger) index;
-(long)longValueAtIndex:(NSUInteger) index;
-(long long)longLongValueAtIndex:(NSUInteger) index;
-(float)floatValueAtIndex:(NSUInteger) index;
-(double)doubleValueAtIndex:(NSUInteger) index;
-(BOOL)boolValueAtIndex:(NSUInteger) index;
-(NSDecimal)decimalValueAtIndex:(NSUInteger) index;
-(NSInteger)integerValueAtIndex:(NSUInteger) index;

-(unsigned char)unsignedCharValueAtIndex:(NSUInteger) index;
-(unsigned short)unsignedShortValueAtIndex:(NSUInteger) index;
-(unsigned int)unsignedIntValueAtIndex:(NSUInteger) index;
-(unsigned long)unsignedLongValueAtIndex:(NSUInteger) index;
-(unsigned long long)unsignedLongLongValueAtIndex:(NSUInteger) index;
-(NSUInteger)unsignedIntegerValueAtIndex:(NSUInteger) index;
-(NSNumber*) numberAtIndex:(NSUInteger)index;

//Editing Features:
-(void)setCharValue:(char)value atIndex:(NSUInteger) index;
-(void)setShortValue:(short)value atIndex:(NSUInteger) index;
-(void)setIntValue:(int)value atIndex:(NSUInteger) index;
-(void)setLongValue:(long)value atIndex:(NSUInteger) index;
-(void)setLongLongValue:(long long)value atIndex:(NSUInteger) index;
-(void)setFloatValue:(float)value atIndex:(NSUInteger) index;
-(void)setDoubleValue:(double)value atIndex:(NSUInteger) index;
-(void)setBoolValue:(BOOL)value atIndex:(NSUInteger) index;
-(void)setIntegerValue:(NSInteger)value atIndex:(NSUInteger) index;

-(void)setUnsignedCharValue:(unsigned char)value atIndex:(NSUInteger) index;
-(void)setUnsignedShortValue:(unsigned short)value atIndex:(NSUInteger) index;
-(void)setUnsignedIntValue:(unsigned int)value atIndex:(NSUInteger) index;
-(void)setUnsignedLongValue:(unsigned long)value atIndex:(NSUInteger) index;
-(void)setUnsignedLongLongValue:(unsigned long long)value atIndex:(NSUInteger) index;
-(void)setUnsignedIntegerValue:(NSUInteger)value atIndex:(NSUInteger) index;
-(void)setNumber:(NSNumber*)value atIndex:(NSUInteger)index;



-(NSString *)stringValueAtIndex: (NSUInteger) index;
-(NSString *)descriptionAtIndex:(NSUInteger)index withLocale:(NSDictionary *)locale;

-(NSArray *)stringArray;//NSArray with NSString elements.
-(NSArray *)descriptionsWithLocalesArray:(NSArray *)locales; //NSArray with NSString elements

-(NSString *)stringValue;//NSString separated by spaces.
-(NSString *)stringValueWithSeparator:(NSString*)separator;

//Methods Type:
-(SCINumberArrayType) numberArrayType;

//Methods for NSarray
//- (NSUInteger)count;
//- (id)objectAtIndex:(NSUInteger)index;


//PRIVATE METHODS DO NOT CALL FOR THEN

-(void *) SCI_dataPointer; //private method


@end

@interface SCINumberArray (SCINumberArrayExtendedInitializers)


-initWithCount:(NSUInteger)count andBools:(BOOL) firstBool , ...;
-initWithCount:(NSUInteger)count andChars:(char) firstChar , ...;
-initWithCount:(NSUInteger)count andShorts:(short) firstShort , ...;
-initWithCount:(NSUInteger)count andInts:(int) firstInt , ...;
-initWithCount:(NSUInteger)count andLongs:(long) firstLong , ...;
-initWithCount:(NSUInteger)count andLongLongs:(long long) firstLongLong , ...;
-initWithCount:(NSUInteger)count andFloats:(float) firstFloat , ...;
-initWithCount:(NSUInteger)count andDoubles:(double) firstDouble , ...;
-initWithCount:(NSUInteger)count andIntegers:(NSInteger) firstInteger , ...;

-initWithCount:(NSUInteger)count andUnsignedChars:(unsigned char) firstUnsignedChar , ...;
-initWithCount:(NSUInteger)count andUnsignedShorts:(unsigned short) firstUnsignedShort , ...;
-initWithCount:(NSUInteger)count andUnsignedInts:(unsigned int) firstUnsignedInt , ...;
-initWithCount:(NSUInteger)count andUnsignedLongs:(unsigned long) firstUnsignedLong , ...;
-initWithCount:(NSUInteger)count andUnsignedLongLongs:(unsigned long long) firstUnsignedLongLong , ...;
-initWithCount:(NSUInteger)count andUnsignedIntegers:(NSUInteger) firstUnsignedInteger , ...;



-initWithString:(NSString*)numberString separator:(NSString*)separator;
-initWithString:(NSString*)numberString;

-initWithBoolsFromString:(NSString*)numberString;
-initWithBoolsFromString:(NSString*)numberString separator:(NSString*)separator;


-initWithCharsFromString:(NSString*)numberString;
-initWithCharsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithShortsFromString:(NSString*)numberString;
-initWithShortsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithIntsFromString:(NSString*)numberString;
-initWithIntsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithLongsFromString:(NSString*)numberString;
-initWithLongsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithLongLongsFromString:(NSString*)numberString;
-initWithLongLongsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithIntegersFromString:(NSString*)numberString;
-initWithIntegersFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithFloatsFromString:(NSString*)numberString;
-initWithFloatsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithDoublesFromString:(NSString*)numberString;
-initWithDoublesFromString:(NSString*)numberString separator:(NSString*)separator;


-initWithUnsignedCharsFromString:(NSString*)numberString;
-initWithUnsignedCharsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithUnsignedShortsFromString:(NSString*)numberString;
-initWithUnsignedShortsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithUnsignedIntsFromString:(NSString*)numberString;
-initWithUnsignedIntsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithUnsignedLongsFromString:(NSString*)numberString;
-initWithUnsignedLongsFromString:(NSString*)numberString separator:(NSString*)separator;


-initWithUnsignedLongLongsFromString:(NSString*)numberString;
-initWithUnsignedLongLongsFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithUnsignedIntegersFromString:(NSString*)numberString;
-initWithUnsignedIntegersFromString:(NSString*)numberString separator:(NSString*)separator;

-initWithString:(NSString*)numberString type:(SCINumberArrayType)type;
-initWithString:(NSString*)numberString separator:(NSString*)separator type:(SCINumberArrayType)type;







+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andBools:(BOOL) firstBool , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andChars:(char) firstChar , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andShorts:(short) firstShort , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andInts:(int) firstInt , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andLongs:(long) firstLong , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andLongLongs:(long long) firstLongLong , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andFloats:(float) firstFloat , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andDoubles:(double) firstDouble , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andIntegers:(NSInteger) firstInteger , ...;

+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andUnsignedChars:(unsigned char) firstUnsignedChar , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andUnsignedShorts:(unsigned short) firstUnsignedShort , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andUnsignedInts:(unsigned int) firstUnsignedInt , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andUnsignedLongs:(unsigned long) firstUnsignedLong , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andUnsignedLongLongs:(unsigned long long) firstUnsignedLongLong , ...;
+(SCINumberArray *)numberArrayWithCount:(NSUInteger)count andUnsignedIntegers:(NSUInteger) firstUnsignedInteger , ...;





+(SCINumberArray *)numberArrayWithString:(NSString*)numberString separator:(NSString*)separator;
+(SCINumberArray *)numberArrayWithString:(NSString*)numberString;

+(SCINumberArray *)numberArrayWithBoolsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithBoolsFromString:(NSString*)numberString separator:(NSString*)separator;


+(SCINumberArray *)numberArrayWithCharsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithCharsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithShortsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithShortsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithIntsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithIntsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithLongsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithLongsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithLongLongsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithLongLongsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithIntegersFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithIntegersFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithFloatsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithFloatsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithDoublesFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithDoublesFromString:(NSString*)numberString separator:(NSString*)separator;


+(SCINumberArray *)numberArrayWithUnsignedCharsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithUnsignedCharsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithUnsignedShortsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithUnsignedShortsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithUnsignedIntsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithUnsignedIntsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithUnsignedLongsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithUnsignedLongsFromString:(NSString*)numberString separator:(NSString*)separator;


+(SCINumberArray *)numberArrayWithUnsignedLongLongsFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithUnsignedLongLongsFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithUnsignedIntegersFromString:(NSString*)numberString;
+(SCINumberArray *)numberArrayWithUnsignedIntegersFromString:(NSString*)numberString separator:(NSString*)separator;

+(SCINumberArray *)numberArrayWithString:(NSString*)numberString type:(SCINumberArrayType)type;
+(SCINumberArray *)numberArrayWithString:(NSString*)numberString separator:(NSString*)separator type:(SCINumberArrayType)type;


-initWithContentsOfFile:(NSString *)path type:(SCINumberArrayType)type;
-initWithContentsOfURL:(NSURL *)aURL type:(SCINumberArrayType)type;
+(SCINumberArray*)numverArrayWithContentsOfFile:(NSString *)path type:(SCINumberArrayType)type;
+(SCINumberArray*)numverArrayWithContentsOfURL:(NSURL *)aURL type:(SCINumberArrayType)type;


@end

