//
//  SCIVector2d.h
//  SciObjects
//
//  Created by Filipi Nascimento Silva on 03/19/2008.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SciObjects_consts.h"
#import "SCIVector.h"
@class SCIVector2d_initializer;

@interface SCIVector2d : NSArray <SCIVector,NSCoding, NSCopying>{

}
#pragma mark Initializers
-initWithX:(NSNumber*)newX Y:(NSNumber*)newY;
-initWithXY:(SCIVector2d*)newXY;
-initWithVector2d:(SCIVector2d*)newVector2d;
-initWithArray:(NSArray *)array;

-initWithX:(NSNumber*)newX Y:(NSNumber*)newY precision:(SCIVectorPrecision)precision;
-initWithXY:(SCIVector2d*)newXY precision:(SCIVectorPrecision)precision;
-initWithVector2d:(SCIVector2d*)newVector2d precision:(SCIVectorPrecision)precision;
-initWithArray:(NSArray *)array precision:(SCIVectorPrecision)precision;

#pragma mark Creators

+(SCIVector2d*)vector2dWithX:(NSNumber*)newX Y:(NSNumber*)newY;
+(SCIVector2d*)vector2dWithX:(NSNumber*)newX Y:(NSNumber*)newY precision:(SCIVectorPrecision)precision;
+(SCIVector2d*)vector2dWithXY:(SCIVector2d*)newXY;
+(SCIVector2d*)vector2dWithXY:(SCIVector2d*)newXY precision:(SCIVectorPrecision)precision;
+(SCIVector2d*)vector2dWithVector2d:(SCIVector2d*)newVector2d;
+(SCIVector2d*)vector2dWithVector2d:(SCIVector2d*)newVector2d precision:(SCIVectorPrecision)precision;

+(SCIVector2d*)vector2dWithArray:(NSArray *)array;
+(SCIVector2d*)vector2dWithArray:(NSArray *)array precision:(SCIVectorPrecision)precision;
+(SCIVector2d*)zero;
+(SCIVector2d*)one;

#pragma mark Get accessors

-(NSNumber*)x; 
-(NSNumber*)y;
-(SCIVector2d*)xy;
-(SCIVector2d*)yx;
-(SCIVector2d*)xx;
-(SCIVector2d*)yy;

#pragma mark Set accessors

-(void)setXY:(SCIVector2d*)newXY;
-(void)setX:(NSNumber*)newX;
-(void)setY:(NSNumber*)newY;
-(void)setX:(NSNumber*)newX Y:(NSNumber*)newY;

#pragma mark Operations
-(NSNumber *)magnitude;
-(NSNumber *)squaredMagnitude;
-(NSNumber *)argument;

-(SCIVector2d *)versor;
-(SCIVector2d *)negative;
-(SCIVector2d *)absolute;

-(SCIVector2d *)add:(SCIVector2d *)vector;
-(SCIVector2d *)sub:(SCIVector2d *)vector;
-(SCIVector2d *)mul:(SCIVector2d *)vector;
-(SCIVector2d *)div:(SCIVector2d *)vector;

-(SCIVector2d *)addScalar:(NSNumber *)number;
-(SCIVector2d *)subScalar:(NSNumber *)number;
-(SCIVector2d *)mulScalar:(NSNumber *)number;
-(SCIVector2d *)divScalar:(NSNumber *)number;

-(SCIVector2d *)dot:(SCIVector2d *)vector;
//-(SCIVector3d *)cross:(SCIVector2d *);


#pragma mark SelfOperations
-(SCIVector2d *)normalize; //selfVersor
-(SCIVector2d *)negate; //selfNegative
-(SCIVector2d *)selfAbsolute;

-(SCIVector2d *)selfAdd:(SCIVector2d *)vector;
-(SCIVector2d *)selfSub:(SCIVector2d *)vector;
-(SCIVector2d *)selfMul:(SCIVector2d *)vector;
-(SCIVector2d *)selfDiv:(SCIVector2d *)vector;

-(SCIVector2d *)selfAddScalar:(NSNumber *)number;
-(SCIVector2d *)selfSubScalar:(NSNumber *)number;
-(SCIVector2d *)selfMulScalar:(NSNumber *)number;
-(SCIVector2d *)selfDivScalar:(NSNumber *)number;

-(SCIVector2d *)selfDot:(SCIVector2d *)vector;

#pragma mark StringMethods
-initWithString:(NSString*)numberString separator:(NSString*)separator precision:(SCIVectorPrecision)precision;
-initWithString:(NSString*)numberString precision:(SCIVectorPrecision)precision;
-initWithString:(NSString*)numberString separator:(NSString*)separator;
-initWithString:(NSString*)numberString;

+vector2dWithString:(NSString*)numberString separator:(NSString*)separator precision:(SCIVectorPrecision)precision;
+vector2dWithString:(NSString*)numberString precision:(SCIVectorPrecision)precision;
+vector2dWithString:(NSString*)numberString separator:(NSString*)separator;
+vector2dWithString:(NSString*)numberString;


-(NSString*) stringValue;
-(NSString*) stringValueWithSeparator:(NSString*)separator;

#pragma mark FileMethods
//-initWithContentsOfFile:(NSString *)path type:(SCINumberArrayType)type;
//-initWithContentsOfURL:(NSURL *)aURL type:(SCINumberArrayType)type;
//+(SCIVector2d*)vector2dWithContentsOfFile:(NSString *)path type:(SCINumberArrayType)type;
//+(SCIVector2d*)vector2dWithContentsOfURL:(NSURL *)aURL type:(SCINumberArrayType)type;


@end

#pragma mark -
@interface SCIVector2d (SCIVector2dPrimitiveAdditions)
#pragma mark Double Methods Additions:
#pragma mark Double Initializers:

-initWithXd:(double)newXd Yd:(double)newYd;
-initWithXYd:(double*)newXYd;
-initWithDoubles:(double*)newDoubles;

#pragma mark Double Creators:

+(SCIVector2d*)vector2dWithXd:(double)newXd Yd:(double)newYd;
+(SCIVector2d*)vector2dWithXYd:(double*)newXYd;
+(SCIVector2d*)vector2dWithDoubles:(double*)newDoubles;
+(SCIVector2d*)zerod;
+(SCIVector2d*)oned;

#pragma mark Double get accessors

-(double)xd; 
-(double)yd;
-(double*)xyd;
-(double*)yxd;
-(double*)xxd;
-(double*)yyd;

#pragma mark Double set accessors

-(void)setXYd:(double*)newXYd;
-(void)setXd:(double)newXd;
-(void)setYd:(double)newYd;
-(void)setXd:(double)newXd Yd:(double)newYd;


#pragma mark Float Methods Additions:
#pragma mark Float Initializers:

-initWithXf:(float)newXf Yf:(float)newYf;
-initWithXYf:(float*)newXYf;
-initWithFloats:(float*)newFloats;

#pragma mark Float Creators:

+(SCIVector2d*)vector2dWithXf:(float)newXf Yf:(float)newYf;
+(SCIVector2d*)vector2dWithXYf:(float*)newXYf;
+(SCIVector2d*)vector2dWithFloats:(float*)newFloats;
+(SCIVector2d*)zerof;
+(SCIVector2d*)onef;

#pragma mark Float get accessors:

-(float)xf; 
-(float)yf;
-(float*)xyf;
-(float*)yxf;
-(float*)xxf;
-(float*)yyf;

#pragma mark Float set accessors:

-(void)setXYf:(float*)newXYf;
-(void)setXf:(float)newXf;
-(void)setYf:(float)newYf;
-(void)setXf:(float)newXf Yf:(float)newYf;



#pragma mark Double Operations
-(double)doubleMagnitude;
-(double)doubleSquaredMagnitude;
-(double)doubleArgument;

-(SCIVector2d *)addDouble:(double)doubleNumber;
-(SCIVector2d *)subDouble:(double)doubleNumber;
-(SCIVector2d *)mulDouble:(double)doubleNumber;
-(SCIVector2d *)divDouble:(double)doubleNumber;


#pragma mark Double SelfOperations

-(SCIVector2d *)selfAddDouble:(double)doubleNumber;
-(SCIVector2d *)selfSubDouble:(double)doubleNumber;
-(SCIVector2d *)selfMulDouble:(double)doubleNumber;
-(SCIVector2d *)selfDivDouble:(double)doubleNumber;

#pragma mark Float Operations
-(float)floatMagnitude;
-(float)floatSquaredMagnitude;
-(float)floatArgument;

-(SCIVector2d *)addFloat:(float)floatNumber;
-(SCIVector2d *)subFloat:(float)floatNumber;
-(SCIVector2d *)mulFloat:(float)floatNumber;
-(SCIVector2d *)divFloat:(float)floatNumber;


#pragma mark Float SelfOperations

-(SCIVector2d *)selfAddFloat:(float)floatNumber;
-(SCIVector2d *)selfSubFloat:(float)floatNumber;
-(SCIVector2d *)selfMulFloat:(float)floatNumber;
-(SCIVector2d *)selfDivFloat:(float)floatNumber;


@end